import { useCallback, useEffect, useMemo, useRef } from "react";
import { Platform, StyleSheet, useColorScheme } from "react-native";
import NativeModule from "./NativeRNBootSplash";
export function hide(config = {}) {
  const {
    fade = false
  } = config;
  return NativeModule.hide(fade).then(() => {});
}
export function isVisible() {
  return NativeModule.isVisible();
}
export function useHideAnimation(config) {
  const {
    manifest,
    logo: logoSrc,
    darkLogo: darkLogoSrc,
    brand: brandSrc,
    darkBrand: darkBrandSrc,
    animate,
    canStartHideAnimation = true,
    statusBarTranslucent = false,
    navigationBarTranslucent = false
  } = config;
  const skipLogo = logoSrc == null;
  const skipBrand = manifest.brand == null || brandSrc == null;
  const logoWidth = manifest.logo.width;
  const logoHeight = manifest.logo.height;
  const brandBottom = manifest.brand?.bottom;
  const brandWidth = manifest.brand?.width;
  const brandHeight = manifest.brand?.height;
  const colorScheme = useColorScheme();
  const backgroundColor = colorScheme === "dark" && manifest.darkBackground != null ? manifest.darkBackground : manifest.background;
  const logoFinalSrc = skipLogo ? undefined : colorScheme === "dark" && darkLogoSrc != null ? darkLogoSrc : logoSrc;
  const brandFinalSrc = skipBrand ? undefined : colorScheme === "dark" && darkBrandSrc != null ? darkBrandSrc : brandSrc;
  const animateFn = useRef(animate);
  const layoutReady = useRef(false);
  const logoReady = useRef(skipLogo);
  const brandReady = useRef(skipBrand);
  const animateHasBeenCalled = useRef(false);
  useEffect(() => {
    animateFn.current = animate;
  });
  const maybeRunAnimate = useCallback(() => {
    if (layoutReady.current && logoReady.current && brandReady.current && !animateHasBeenCalled.current && canStartHideAnimation) {
      animateHasBeenCalled.current = true;
      hide({
        fade: false
      }).then(() => animateFn.current()).catch(() => {});
    }
  }, [canStartHideAnimation]);
  useEffect(() => {
    if (canStartHideAnimation) {
      maybeRunAnimate();
    }
  }, [canStartHideAnimation, maybeRunAnimate]);
  return useMemo(() => {
    const containerStyle = {
      ...StyleSheet.absoluteFillObject,
      backgroundColor,
      alignItems: "center",
      justifyContent: "center"
    };
    const container = {
      style: containerStyle,
      onLayout: () => {
        layoutReady.current = true;
        maybeRunAnimate();
      }
    };
    const logo = logoFinalSrc == null ? {
      source: -1
    } : {
      fadeDuration: 0,
      resizeMode: "contain",
      source: logoFinalSrc,
      style: {
        width: logoWidth,
        height: logoHeight
      },
      onLoadEnd: () => {
        logoReady.current = true;
        maybeRunAnimate();
      }
    };
    const brand = brandFinalSrc == null ? {
      source: -1
    } : {
      fadeDuration: 0,
      resizeMode: "contain",
      source: brandFinalSrc,
      style: {
        position: "absolute",
        bottom: Platform.OS === "web" ? 60 : brandBottom,
        width: brandWidth,
        height: brandHeight
      },
      onLoadEnd: () => {
        brandReady.current = true;
        maybeRunAnimate();
      }
    };
    if (Platform.OS !== "android") {
      return {
        container,
        logo,
        brand
      };
    }
    const {
      logoSizeRatio,
      navigationBarHeight,
      statusBarHeight
    } = NativeModule.getConstants();
    return {
      container: {
        ...container,
        style: {
          ...containerStyle,
          marginTop: statusBarTranslucent ? undefined : -statusBarHeight,
          marginBottom: navigationBarTranslucent ? undefined : -navigationBarHeight
        }
      },
      logo: {
        ...logo,
        style: {
          width: logoWidth * logoSizeRatio,
          height: logoHeight * logoSizeRatio
        }
      },
      brand
    };
  }, [maybeRunAnimate, logoWidth, logoHeight, brandBottom, brandWidth, brandHeight, backgroundColor, logoFinalSrc, brandFinalSrc, statusBarTranslucent, navigationBarTranslucent]);
}
export default {
  hide,
  isVisible,
  useHideAnimation
};
//# sourceMappingURL=index.js.map